@@ -2,7 +2,7 @@
 * socket.cpp: socket management
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: socket.cpp,v 1.11 2002/04/27 01:15:57 asmax Exp $
+* $Id: socket.cpp,v 1.17 2002/10/07 15:01:21 sam Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *          Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
@@ -73,20 +73,12 @@
 // Plateform specific issues
 //------------------------------------------------------------------------------
 #if defined WIN32
-#define ERR_INTR        WSAEINTR
-#define bufftype        char
-#define socklen         int
-
+#   define ERR_INTR        WSAEINTR
 #else
-#define ERR_INTR        EINTR
-#define bufftype        byte
-#define socklen         socklen_t
-
+#   define ERR_INTR        EINTR
 #endif
 
 
-
-
 //******************************************************************************
 // E_InetAddr class
 //******************************************************************************
@@ -161,17 +153,17 @@ C_SocketBuff::~C_SocketBuff()
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-void C_SocketBuff::AssignSlot(unsigned int iSlot, byte* pBuff,
+void C_SocketBuff::AssignSlot(unsigned int iSlot, char* pBuff,
                               unsigned int iBuffSize)
 {
   ASSERT(iSlot < m_iSlotCount);
   ASSERT(pBuff);
   
 #ifdef STRUCT_IOVEC_IN_SYS_UIO_H
-  m_pBuffers[iSlot].iov_base = (bufftype*)pBuff;
+  m_pBuffers[iSlot].iov_base = pBuff;
   m_pBuffers[iSlot].iov_len = iBuffSize;
 #elif defined WIN32
-  m_pBuffers[iSlot].buf = (bufftype*)pBuff;
+  m_pBuffers[iSlot].buf = pBuff;
   m_pBuffers[iSlot].len = iBuffSize;
 #endif
 }
@@ -184,15 +176,15 @@ void C_SocketBuff::AssignSlot(unsigned int iSlot, byte* pBuff,
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-void C_SocketBuff::SetSlotBuff(unsigned int iSlot, byte* pBuff)
+void C_SocketBuff::SetSlotBuff(unsigned int iSlot, char* pBuff)
 {
   ASSERT(iSlot < m_iSlotCount);
   ASSERT(pBuff);
   
 #ifdef STRUCT_IOVEC_IN_SYS_UIO_H
-  m_pBuffers[iSlot].iov_base = (bufftype*)pBuff;
+  m_pBuffers[iSlot].iov_base = pBuff;
 #elif defined WIN32
-  m_pBuffers[iSlot].buf = (bufftype*)pBuff;
+  m_pBuffers[iSlot].buf = pBuff;
 #endif
 }
 
@@ -371,10 +363,21 @@ void C_Socket::Close()
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-void C_Socket::SetOption(int iLevel, int iOpt, const void* pOptVal, int iOptLen)
+int C_Socket::GetDomain()
 {
-  int iRc = setsockopt(m_hSocket, iLevel, iOpt, (bufftype*)pOptVal,
-                       (socklen)iOptLen);
+	// returns the address family of this socket
+	// or AF_UNSPEC (set in constructor)
+	return m_iDomain;
+}
+
+
+//------------------------------------------------------------------------------
+//
+//------------------------------------------------------------------------------
+void C_Socket::SetOption(int iLevel, int iOpt, const char* pOptVal, socklen_t iOptLen)
+{
+  int iRc = setsockopt(m_hSocket, iLevel, iOpt, pOptVal,
+                       iOptLen);
 
   if(iRc)
   {
@@ -387,10 +390,10 @@ void C_Socket::SetOption(int iLevel, int iOpt, const void* pOptVal, int iOptLen)
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-void C_Socket::GetOption(int iLevel, int iOpt, void* pOptVal, int* pOptLen)
+void C_Socket::GetOption(int iLevel, int iOpt, char* pOptVal, socklen_t* pOptLen)
 {
-  int iRc = getsockopt(m_hSocket, iLevel, iOpt, (bufftype*)pOptVal,
-                       (socklen*)pOptLen);
+  int iRc = getsockopt(m_hSocket, iLevel, iOpt, pOptVal,
+                       pOptLen);
 
   if(iRc)
   {
@@ -407,7 +410,7 @@ void C_Socket::Bind(const C_String& strIntf, const C_String& strPort)
 {
   // Try to resolve the given address
   struct sockaddr* pAddr;
-  socklen iAddrLen;
+  int iAddrLen;
 
   if(m_iDomain == AF_INET)
   {
@@ -465,7 +468,7 @@ void C_Socket::Connect(const C_String& strPeer, const C_String& strPort)
 {
   // Try to resolve the given address
   struct sockaddr* pAddr;
-  socklen iAddrLen;
+  int iAddrLen;
 
   if(m_iDomain == AF_INET)
   {
@@ -543,7 +546,7 @@ C_Socket* C_Socket::Accept()
   if(m_iDomain == AF_INET)
   {
     struct sockaddr_in sPeer;
-    socklen iAddrLen = sizeof(sPeer);
+    socklen_t iAddrLen = sizeof(sPeer);
   
     iRc = accept(m_hSocket, (struct sockaddr*)&sPeer, &iAddrLen);
   }
@@ -551,7 +554,7 @@ C_Socket* C_Socket::Accept()
   else if(m_iDomain == AF_INET6)
   {
     struct sockaddr_in6 sPeer;
-    socklen iAddrLen = sizeof(sPeer);
+    socklen_t iAddrLen = sizeof(sPeer);
 
     iRc = accept(m_hSocket, (struct sockaddr*)&sPeer, &iAddrLen);
   }
@@ -587,12 +590,12 @@ C_Socket* C_Socket::Accept()
 // a timeout set to 0
 // Returns the number of byte read or throw an exception if an error occurs
 //------------------------------------------------------------------------------
-int C_Socket::Read(byte* pBuff, int iBuffLen, int iFlags)
+int C_Socket::Read(char* pBuff, int iBuffLen, int iFlags)
 {
   ASSERT(pBuff);
 
   // Wait for data until something is received
-  int iRc = recv(m_hSocket, (bufftype*)pBuff, iBuffLen, iFlags);
+  int iRc = recv(m_hSocket, pBuff, iBuffLen, iFlags);
 
   if(iRc < 0)
   {
@@ -606,7 +609,7 @@ int C_Socket::Read(byte* pBuff, int iBuffLen, int iFlags)
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-int C_Socket::ReadTimeOut(byte* pBuff, int iBuffLen, int iTimeOut, int iFlags)
+int C_Socket::ReadTimeOut(char* pBuff, int iBuffLen, int iTimeOut, int iFlags)
 {
   ASSERT(pBuff);
   
@@ -631,7 +634,7 @@ int C_Socket::ReadTimeOut(byte* pBuff, int iBuffLen, int iTimeOut, int iFlags)
     if (iRc > 0)
     {
       // Data were received before timeout
-      iRc = recv(m_hSocket, (bufftype*)pBuff, iBuffLen, iFlags);
+      iRc = recv(m_hSocket, pBuff, iBuffLen, iFlags);
     }
     else
     {
@@ -662,12 +665,12 @@ int C_Socket::ReadTimeOut(byte* pBuff, int iBuffLen, int iTimeOut, int iFlags)
 // a timeout set to 0
 // Returns the number of byte written or throw an exception if an error occurs
 //------------------------------------------------------------------------------
-int C_Socket::Write(const byte* pBuff, int iBuffLen, int iFlags)
+int C_Socket::Write(const char* pBuff, int iBuffLen, int iFlags)
 {
   ASSERT(pBuff);
 
   // Wait until the data could be sent
-  int iRc = send(m_hSocket, (bufftype*)pBuff, iBuffLen, iFlags);
+  int iRc = send(m_hSocket, pBuff, iBuffLen, iFlags);
 
   if(iRc < 0)
   {
@@ -681,7 +684,8 @@ int C_Socket::Write(const byte* pBuff, int iBuffLen, int iFlags)
 //------------------------------------------------------------------------------
 //
 //------------------------------------------------------------------------------
-int C_Socket::Send(const C_SocketBuff& cBuffers, unsigned int iSlotCount)
+int C_Socket::Send(const C_SocketBuff& cBuffers, unsigned int iSlotCount,
+                                                 unsigned int iOffset)
 { 
   ASSERT(iSlotCount <= cBuffers.GetSlotCount());
 
@@ -689,11 +693,11 @@ int C_Socket::Send(const C_SocketBuff& cBuffers, unsigned int iSlotCount)
   
   // Send the data contained in the buffers
 #ifdef STRUCT_IOVEC_IN_SYS_UIO_H
-  int iRc = writev(m_hSocket, pBuff, iSlotCount);
-
+  int iRc = writev(m_hSocket, &pBuff[iOffset], iSlotCount);
 #elif defined WIN32
   unsigned long count;
-  int iRc = WSASend(m_hSocket, pBuff, iSlotCount, &count, NULL, NULL, NULL);
+  int iRc = WSASend(m_hSocket, pBuff+iOffset,
+                    iSlotCount, &count, 0, NULL, NULL);
 #endif
 
   if(iRc < 0)
@@ -710,7 +714,7 @@ int C_Socket::Send(const C_SocketBuff& cBuffers, unsigned int iSlotCount)
 //------------------------------------------------------------------------------
 int C_Socket::Send(const C_SocketBuff& cBuffers)
 {
-  return Send(cBuffers, cBuffers.GetSlotCount());
+  return Send(cBuffers, cBuffers.GetSlotCount(),0);
 }
 
 
@@ -718,12 +722,12 @@ int C_Socket::Send(const C_SocketBuff& cBuffers)
 //
 //------------------------------------------------------------------------------
 int C_Socket::WriteTo(C_Inet4Addr& cPeerAddr,
-                      const byte* pBuff, int iBuffLen, int iFlags)
+                      const char* pBuff, int iBuffLen, int iFlags)
 {
   ASSERT(pBuff);
   ASSERT(m_iDomain == AF_INET);
 
-  int iRc = sendto(m_hSocket, (bufftype*)pBuff, iBuffLen, iFlags,
+  int iRc = sendto(m_hSocket, pBuff, iBuffLen, iFlags,
                    (const sockaddr*)cPeerAddr.GetInetAddr(),
                    sizeof(*cPeerAddr.GetInetAddr()));
 
@@ -741,12 +745,12 @@ int C_Socket::WriteTo(C_Inet4Addr& cPeerAddr,
 //------------------------------------------------------------------------------
 #ifdef HAVE_IPV6
 int C_Socket::WriteTo(C_Inet6Addr& cPeerAddr,
-                      const byte* pBuff, int iBuffLen, int iFlags)
+                      const char* pBuff, int iBuffLen, int iFlags)
 {
   ASSERT(pBuff);
   ASSERT(m_iDomain == AF_INET6);
 
-  int iRc = sendto(m_hSocket, (bufftype*)pBuff, iBuffLen, iFlags,
+  int iRc = sendto(m_hSocket, pBuff, iBuffLen, iFlags,
                    (const sockaddr*)cPeerAddr.GetInetAddr(),
                    sizeof(*cPeerAddr.GetInetAddr()));
 
@@ -790,10 +794,10 @@ int C_Socket::Recv(C_SocketBuff& cBuffers, unsigned int iSlotCount)
 {
   ASSERT(iSlotCount <= cBuffers.GetSlotCount());
 
+#ifdef STRUCT_IOVEC_IN_SYS_UIO_H
   C_SocketBuff::sock_buff* pBuff = cBuffers.GetRawBuffers();
   
   // Read the data and put them in the buffers
-#ifdef STRUCT_IOVEC_IN_SYS_UIO_H
   int iRc = readv(m_hSocket, pBuff, iSlotCount);
 
 #elif defined WIN32
@@ -839,7 +843,7 @@ C_String C_Socket::GetLocalName() const
   if(m_iDomain == AF_INET)
   {
     struct sockaddr_in saIntf;
-    socklen iNameLen = sizeof(saIntf);
+    socklen_t iNameLen = sizeof(saIntf);
     int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
     ASSERT(iNameLen == sizeof(saIntf));
     if(iRc)
@@ -854,7 +858,7 @@ C_String C_Socket::GetLocalName() const
   else if(m_iDomain == AF_INET6)
   {
     struct sockaddr_in6 saIntf;
-    socklen iNameLen = sizeof(saIntf);
+    socklen_t iNameLen = sizeof(saIntf);
     int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
     ASSERT(iNameLen == sizeof(saIntf));
     if(iRc)
@@ -892,7 +896,7 @@ C_String C_Socket::GetLocalPort() const
   if(m_iDomain == AF_INET)
   {
     struct sockaddr_in saIntf;
-    socklen iNameLen = sizeof(saIntf);
+    socklen_t iNameLen = sizeof(saIntf);
     int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
     ASSERT(iNameLen == sizeof(saIntf));
     if(iRc)
@@ -907,7 +911,7 @@ C_String C_Socket::GetLocalPort() const
   else if(m_iDomain == AF_INET6)
   {
     struct sockaddr_in6 saIntf;
-    socklen iNameLen = sizeof(saIntf);
+    socklen_t iNameLen = sizeof(saIntf);
     int iRc = getsockname(m_hSocket, (struct sockaddr*)&saIntf, &iNameLen);
     ASSERT(iNameLen == sizeof(saIntf));
     if(iRc)
@@ -938,7 +942,7 @@ C_String C_Socket::GetPeerName() const
   if(m_iDomain == AF_INET)
   {
     struct sockaddr_in saPeer;
-    socklen iNameLen = sizeof(saPeer);
+    socklen_t iNameLen = sizeof(saPeer);
     int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
     ASSERT(iNameLen == sizeof(saPeer));
     if(iRc)
@@ -953,7 +957,7 @@ C_String C_Socket::GetPeerName() const
   else if(m_iDomain == AF_INET6)
   {
     struct sockaddr_in6 saPeer;
-    socklen iNameLen = sizeof(saPeer);
+    socklen_t iNameLen = sizeof(saPeer);
     int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
     ASSERT(iNameLen == sizeof(saPeer));
     if(iRc)
@@ -991,7 +995,7 @@ C_String C_Socket::GetPeerPort() const
   if(m_iDomain == AF_INET)
   {
     struct sockaddr_in saPeer;
-    socklen iNameLen = sizeof(saPeer);
+    socklen_t iNameLen = sizeof(saPeer);
     int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
     ASSERT(iNameLen == sizeof(saPeer));
     if(iRc)
@@ -1006,7 +1010,7 @@ C_String C_Socket::GetPeerPort() const
   if(m_iDomain == AF_INET6)
   {
     struct sockaddr_in6 saPeer;
-    socklen iNameLen = sizeof(saPeer);
+    socklen_t iNameLen = sizeof(saPeer);
     int iRc = getpeername(m_hSocket, (struct sockaddr*)&saPeer, &iNameLen);
     ASSERT(iNameLen == sizeof(saPeer));
     if(iRc)
@@ -1225,8 +1229,8 @@ C_Socket* C_SocketPool::InternalMonitor(struct timeval* pTimeOut)
     FD_ZERO(&sSendSet);
     for(unsigned int j = 0; j < m_vSendSockets.Size(); j++)
     {
-      if(m_hHigherSocket < m_vRecvSockets[j].m_hSocket)
-        m_hHigherSocket = m_vRecvSockets[j].m_hSocket;
+      if(m_hHigherSocket < m_vSendSockets[j].m_hSocket)
+        m_hHigherSocket = m_vSendSockets[j].m_hSocket;
 
       FD_SET(m_vSendSockets[j].m_hSocket, &sSendSet);
     }
@@ -1235,8 +1239,8 @@ C_Socket* C_SocketPool::InternalMonitor(struct timeval* pTimeOut)
     FD_ZERO(&sExceptSet);
     for(unsigned int k = 0; k < m_vExceptSockets.Size(); k++)
     {
-      if(m_hHigherSocket < m_vRecvSockets[k].m_hSocket)
-        m_hHigherSocket = m_vRecvSockets[k].m_hSocket;
+      if(m_hHigherSocket < m_vExceptSockets[k].m_hSocket)
+        m_hHigherSocket = m_vExceptSockets[k].m_hSocket;
 
       FD_SET(m_vExceptSockets[k].m_hSocket, &sExceptSet);
     }
